#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <stdlib.h> 
#include <stdio.h>
#include <math.h> 
#include <util/delay.h>  
#include <avr/sleep.h>
#include <string.h>
#include "lcd_lib.h"
#include <stdint.h>
#include "PCM_Sound.h"
#include <inttypes.h>

// Need this for all the things
#define F_CPU 16000000UL

// Value for ADC conversion storage
int Ain = 0;

// Because we're lazy
#define begin {
#define end   }

// Vars to debounce hits
char int0_go;
char int0_cnt=0;
char int1_go;
char int1_cnt=0;

// Vars for Wav Playback
volatile int sampleCount = 0;
volatile char play_en=0;
volatile char inByte = 0;

// Vars for Microphone to output Amplitude Conversion
volatile char micIn;
volatile char micInCorrect;
volatile char amplitudeScale=0;

// Prototype for get ADC value function
int getConv(void);

// Default strings for LCD.
const int8_t LCD_initialize[] PROGMEM = "Drums Anywhere\0";
const int8_t Voice1[] PROGMEM = "1: Clap\0";
const int8_t Voice2[] PROGMEM = "2: Snare\0";


//**********************************************************
// --- external interrupt ISR for Box 1----------------------
ISR(INT0_vect) {
	if (int0_go) {
		EIMSK &= 0x00;
		ADMUX = (1<<ADLAR) | (1<<REFS0) ;
		ADCSRA |= (1<<ADSC) ;
		int0_go = 0;
		TCNT0=0;
		play_en = 1;
		inByte = 1;
		EIMSK = (1<<INT0) | (1<<INT1);
	}
}

//**********************************************************
// --- external interrupt ISR for Box 2----------------------
ISR(INT1_vect) {
	if (int1_go) {
		EIMSK &= 0x00;
		ADMUX = ((1<<ADLAR) | (1<<REFS0)) + 7 ;
		ADCSRA |= (1<<ADSC) ;
		int1_go = 0;
		play_en = 1;
		TCNT2=0;
		inByte = 2;
		EIMSK = (1<<INT0) | (1<<INT1);
	}
}

//**********************************************************
//timer 0 OCRA ISR
// Provides safety timer for Box 1 interrupts
ISR (TIMER0_COMPA_vect)
begin     
	if (!int0_go) {
		int0_cnt++;
		if (int0_cnt == 19) {
			int0_go=1;
			int0_cnt=0;
		}
	}
end

//**********************************************************
//timer 2 OCRA ISR
// Provides safety timer for Box 2 interrupts
ISR (TIMER2_COMPA_vect)
begin     
	if (!int1_go) {
		int1_cnt++;
		if (int1_cnt == 19) {
			int1_go=1;
			int1_cnt=0;
		}
	}
end

//**********************************************************
//timer 1 overflow ISR
// Times output of sound, reads out sound based on
// most recently hit box. Also calculates amplitude
// Runs on 22kHz timer
ISR(TIMER1_OVF_vect) {
	if (play_en) {
		micIn = getConv();
		if (micIn-40 < 0) micInCorrect = 0;
		else micInCorrect= micIn-40;
		amplitudeScale = 7-((micInCorrect)/20);
		if (inByte == 1) {
			OCR1A = pgm_read_byte(&Clap_PCM[sampleCount])>>amplitudeScale;
			sampleCount++;
			if (sampleCount >= Clap_Len) {
				play_en=0;
				sampleCount=0;
			}
		}
		else if (inByte == 2) {
			OCR1B = pgm_read_byte(&Snare_PCM[sampleCount])>>amplitudeScale;
			sampleCount++;
			if (sampleCount >= Snare_Len) {
				play_en=0;
				sampleCount= 0;
			}
		}
	}
}

//**********************************************************
// LCD setup
void init_lcd(void)
begin
    LCDinit();  //initialize the display
    LCDcursorOFF();
    LCDclr();               //clear the display
    LCDGotoXY(0,0);
	CopyStringtoLCD(LCD_initialize, 0, 0);
	CopyStringtoLCD(Voice1, 0, 2);
	CopyStringtoLCD(Voice2, 0, 3);
end

//**********************************************************
// Initialize all timers, ports, variables, and the LCD
void initialize(void) 
begin
	init_lcd();
	DDRA &= 0x00;
	DDRD |= (1<<PIND5) | (1<<PIND4);
	PORTD &= 0x00;
	DDRB |= (1<<PINB2) | (1<<PINB3);

	//init the A to D converter 
	//channel zero/ left adj
	ADMUX = ((1<<ADLAR) | (1<<REFS0)) +7 ; 
	 
	//enable ADC and set prescalar to 1/128*16MHz=125,000
	//and clear interupt enable
	ADCSRA = (1<<ADEN) +7; 
	
	//initialize external Interrupt (Int0 and Int1)
    //set up INT0
    EIMSK = (1<<INT0) | (1<<INT1); // turn on int0 and int1
    EICRA = 0b00001111 ;       // rising edge

	// Set up Timer0 for trigger 0 debounce
	//set up timer 0 for 1 mSec ticks
    TIMSK0 = 2;     //turn on timer 0 cmp match ISR
    OCR0A = 249;    //set the compare reg to 250 time ticks
    TCCR0B = 0b00000011;    // clock prescalar to 64

	// Set up Timer2 for trigger 1 debounce
	//set up timer 0 for 1 mSec ticks
    TIMSK2 = 2;     //turn on timer 0 cmp match ISR
    OCR2A = 249;    //set the compare reg to 250 time ticks
    TCCR2B = 0b00000011;    // clock prescalar to 64

	// Set up Timer1
    // 16000000 (CPU CLOCK) / 760 (10 Bits) = 21052 samples-per-second
	// Use Fast PWM and both output channels
	TCCR1A |= (1<<COM1A1) | (1<<WGM11) | (1<<COM1B1);
	TCCR1B |= (1<<WGM12) | (1<<WGM13) | (1<<CS10);
	ICR1 = 760-1;
	TIMSK1 = (1<<TOIE1);
	OCR1A = 128;
	OCR1B = 128;
	
	// Initialize playback vars
	play_en=0;
	inByte = 1;
	sampleCount = 0;

	//enable interrupts
	sei();
end

//**********************************************************
// Get the most recently read ADC value.
int getConv(void)
begin
	Ain = ADCH;
  return Ain;
end

//**********************************************************
// Main program loop
int main()
begin	
	initialize();
	char LCDbuffer[20];

	// Loop forever, print most recently read micIn value
	while(1) {
		LCDGotoXY(0,1);
		sprintf(LCDbuffer,"Amp: %i ",micIn);
		LCDstring(LCDbuffer,strlen(LCDbuffer));
		LCDsendChar(' ');
	    LCDsendChar(' ');
	}
end
